@@ -16,7 +16,9 @@
 #' sampler is printed to the screen.  If \code{verbose} is greater than 0 the
 #' iteration number, the \eqn{\beta} vector, and the error variance are
 #' printed to the screen every \code{verbose}th iteration.
-#'
+#' @param reduce.mcmc The number of reduced MCMC iterations for marginal likelihood computations.
+#' If \code{reduce.mcmc = NULL}, \code{mcmc/thin} is used.            
+
 #' @param degree.normal	A null model for degree correction. Users can choose "NULL", "eigen" or "Lsym."
 #' "NULL" is no degree correction. "eigen" is a principal eigen-matrix consisting of
 #' the first eigenvalue and the corresponding eigenvector. "
@@ -36,6 +38,9 @@
 #' printed to the screen with the sampled mean values at every \code{verbose}th iteration.
 #' The default is \code{plotUU = FALSE}.
 #'
+#' @param constant If \code{constant = TRUE}, constant parameter is sampled
+#' and saved in the output as attribute \code{bmat}. Default is \code{constant = FALSE}.
+#'
 #' 
 #' @param b0 The prior mean of \eqn{\beta}. This must be a scalar. The default value is 0.
 #' @param B0 The prior variance of \eqn{\beta}. This must be a scalar.  The default value is 1.
@@ -84,18 +89,21 @@
 
 #' Siddhartha Chib. 1995. ``Marginal Likelihood from the Gibbs Output.''
 #' \emph{Journal of the American Statistical Association}. 90: 1313-1321.
-
+#'
 #' @export
 #' @examples
 #'
 #'    \dontrun{
 #'    set.seed(1973)
 #'
-#'    ## generate an array with two constant blocks
-#'    Y <- MakeBlockNetworkChange(n=10, shape=10, T=40, type ="constant")
-#'    out0 <- NetworkStatic(Y, R=2, mcmc=10, burnin=10,
-#'    verbose=10, UL.Normal = "Orthonormal")
+#'    ## generate an array with three constant blocks
+#'    Y <- MakeBlockNetworkChange(n=10, shape=10, T=10, type ="constant")
+#'    G <- 100 ## Small mcmc scans to save time
+#'    out0 <- NetworkStatic(Y, R=2, mcmc=G, burnin=G, verbose=G)
 #'
+#'    ## recovered latent blocks
+#'    Kmeans(out0, n.cluster=3, main="Recovered Blocks")
+#' 
 #'    ## contour plot of latent node positions
 #'    plotContour(out0)
 #'
@@ -106,9 +114,9 @@
 #'    plotV(out0)
 #'    }
 #'
-NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1, 
+NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1,  reduce.mcmc = NULL,    
                           degree.normal="eigen", UL.Normal = "Orthonormal",
-                          plotUU = FALSE, plotZ = FALSE,
+                          plotUU = FALSE, plotZ = FALSE, constant=FALSE, 
                           b0 = 0, B0 = 1, c0 = NULL, d0 = NULL,
                           u0 = NULL, u1 = NULL, v0 = NULL, v1 = NULL,
                           marginal = FALSE, DIC = FALSE,  Waic=FALSE){
@@ -127,13 +135,22 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     ##
     totiter <- mcmc + burnin
     nstore <- mcmc/thin    
-    reduce.mcmc <- nstore
-    K <- dim(Y)  
+    if(is.null(reduce.mcmc)){
+        reduce.mcmc = mcmc/thin
+    }
+    if(is.na(dim(Y)[3])){
+        Y <- array(Y, dim=c(dim(Y)[1], dim(Y)[2], 1))
+    }
+    K <- dim(Y)
     n <- dim(Y)[1] 
     Z <- Y
     MU.record <- Y*0
     nss <- 0
-    
+
+
+    if(R==1 & UL.Normal == "Orthonormal"|| R==1 & UL.Normal == "Normal"){
+        stop("If R=1, please set UL.Normal=FALSE.")
+    }
     ##
     ## Degree normalization
     ##
@@ -153,7 +170,7 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         }
     }
     ## if Modul
-    gamma.par = 0.5
+    gamma.par = 1
     if(degree.normal == "Modul"){
         for(k in 1:K[3]){
           Yk <- as.matrix(Y[,,k])
@@ -215,16 +232,20 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     }
     
     ## initialize parameters
-    bhat <- mean(c(Z))
+    if(constant){
+        bhat <- mean(c(Z))
+        Zb <- Z - bhat
+    }else{
+        bhat = 0
+        Zb <- Z
+    }
     X <- array(1, dim=c(K, 1))
     p <- dim(X)[4]
     XtX <- prod(K) 
     rm(X)
-    Zb <- Z - bhat
     s2 <- d0
     iVV <- iVU <- diag(R) ; eV <- eU <- rep(0,R) ; 
     
-    
     ##
     ## Model diagnositics
     ##
@@ -235,21 +256,24 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     }
     UTAsingle <-  upper.tri(Z[,,1])          
 
-    cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
-    cat("\t NetworkStatic MCMC Sampler Starts! \n")
-    ## cat("\t function called: ")
-    ## print(call)
-    cat("\t degree normalization: ", degree.normal, "\n")
-    cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
-
+    if(verbose !=0){
+        cat("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
+        cat("    NetworkStatic MCMC Sampler Starts! \n")
+        ## cat("\t function called: ")
+        ## print(call)
+        cat("    degree normalization: ", degree.normal, "\n")
+        cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
+    }
     ## ----------------------------------------------
     ## MCMC loop starts!  
     ## ----------------------------------------------
     for(iter in 1:totiter) {   
 
         ## Step 1: update bhat
-        bhat <- updateb(Z, MU, s2, XtX, b0, B0)
-        Zb <- Z - bhat
+        if(constant){
+            bhat <- updateb(Z, MU, s2, XtX, b0, B0)
+            Zb <- Z - bhat
+        }
         
         ## Step 2: update U
         U <- updateU(K, U, V, R, Zb, s2, eU, iVU)
@@ -290,7 +314,9 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         if (verbose!= 0 &iter %% verbose == 0){
             cat("\n----------------------------------------------",'\n')
             cat("    iteration = ", iter, '\n')
-            cat("    beta = ", bhat,'\n')
+            if(constant){
+                cat("    beta = ", bhat,'\n')
+            }
             cat("    sigma2 = ", s2 ,'\n')
             if(plotZ == TRUE & plotUU == TRUE){
                 par(mfrow=c(1, 2))
@@ -313,7 +339,9 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         if (iter > burnin & (iter-burnin)%%thin == 0){
             nss <- nss+1
             MU.record <- MU.record + MU
-            bmat[(iter-burnin)/thin] <- bhat
+            if(constant){
+                bmat[(iter-burnin)/thin] <- bhat
+            }
             Umat[(iter-burnin)/thin, ] <- as.vector(U)
             Vmat[(iter-burnin)/thin, ] <- as.vector(V)
             eUmat[(iter-burnin)/thin, ] <- eU
@@ -339,7 +367,12 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     logmarglike.upper <- loglike.upper <- NA
     
     ## Prepare Stars
-    bhat.st <- mean(bmat)
+    if(constant){
+        bhat.st <- mean(bmat)
+        Zb.st <- Z - bhat.st
+    }else{
+        Zb.st <- Z
+    }
     Sigma.st <- mean(s2mat)
     eU.st <-apply(eUmat, 2, mean)
     eV.st <- apply(eVmat, 2, mean)
@@ -352,7 +385,6 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     ## }else if(UL.Normal == "Orthonormal"){
     ##     U.st <- GramSchmidt(U.st)
     ## }
-    Zb.st <- Z - bhat.st
     MU.st <- M.U(list(U.st, U.st, L.st))
     EE.st <- c(Zb.st - MU.st) 
     
@@ -364,8 +396,9 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     ## loglike <- sum(sapply(1:K[3], function(t){dnorm(c(Zb.st[,,t]),
     ##                                                mean = c(MU.st[,,t]), sqrt(Sigma.st), log=TRUE)}))
     ## cat("    loglike: ", as.numeric(loglike), "\n")
-    cat("    loglike : ", as.numeric(loglike.upper), "\n")
-    
+    if(verbose !=0){
+        cat("    loglike : ", as.numeric(loglike.upper), "\n")
+    }
     
 ##########################################################################
     ## DIC and Waic
@@ -401,17 +434,19 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         
         ## 
         ## cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
-        cat("    Waic: ", Waic.out[1], "\n")
-        ## cat("lpd: ", Waic.out[3], "\n")
-        ## cat("p_Waic: ", Waic.out[4], "\n")
-        ## cat("elapsed time for m = 0 is ", proc.time() - ptm, "\n")
-        ## cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
-        cat("----------------------------------------------",'\n')
+        if(verbose !=0){
+            cat("    Waic: ", Waic.out[1], "\n")
+            ## cat("lpd: ", Waic.out[3], "\n")
+            ## cat("p_Waic: ", Waic.out[4], "\n")
+            ## cat("elapsed time for m = 0 is ", proc.time() - ptm, "\n")
+            ## cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
+            cat("----------------------------------------------",'\n')
+        }
     }
     
     output <- MU.record/nss
     names(output) <- "MU"
-    rm(MU.record) 
+    ## rm(MU.record) 
 
 ##########################################################################
     ## Marginal Likelihood computation
@@ -431,7 +466,9 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         ##
         density.eU.holder <- matrix(NA, reduce.mcmc)
         for (g in 1:reduce.mcmc){
-            Zbg <- Z - bmat[g]         
+            ## if(constant){
+            ##     Zbg <- Z - bmat[g]
+            ## }
             Vg <- matrix(Vmat[g, ], K[3], R)
             Ug <- matrix(Umat[g,], K[1], R)
 
@@ -453,15 +490,15 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         }
         density.eU <- log(mean(exp(density.eU.holder)))
         if(abs(density.eU) == Inf){
-            cat("    Precision reinforced! \n")
-            print(density.eU.holder)
+            ## cat("    Precision reinforced! \n")
+            ## print(density.eU.holder)
             density.eU <- as.numeric(log(mean(exp(mpfr(density.eU.holder, precBits=53)))))
         }
         
-        cat("\n---------------------------------------------- \n ")
-        cat("Marignal Likelihood Computation Step 1 \n")
-        cat("    density.eU: ", as.numeric(density.eU), "\n")
-        cat("---------------------------------------------- \n ")
+        ## cat("\n---------------------------------------------- \n ")
+        ## cat("Marignal Likelihood Computation Step 1 \n")
+        ## cat("    density.eU: ", as.numeric(density.eU), "\n")
+        ## cat("---------------------------------------------- \n ")
 
         ##
         ## Marginal Step 2: p(iVU.st|Z, eu.st)
@@ -469,8 +506,10 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         density.iVU.holder <- matrix(NA, reduce.mcmc)
         for (g in 1:reduce.mcmc){
             ## Step 1: update bhat
-            bhat <- updateb(Z, MU, s2, XtX, b0, B0)
-            Zb <- Z - bhat
+            if(constant){
+                bhat <- updateb(Z, MU, s2, XtX, b0, B0)
+                Zb <- Z - bhat
+            }
             
             ## Step 2: update U
             U <- updateU(K, U, V, R, Zb, s2, eU.st, iVU)
@@ -520,15 +559,15 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         
         density.iVU <- log(mean(exp(density.iVU.holder)))
         if(abs(density.iVU) == Inf){
-            cat("    Precision reinforced! \n")
-            print(density.iVU.holder)
+            ## cat("    Precision reinforced! \n")
+            ## print(density.iVU.holder)
             density.iVU <- as.numeric(log(mean(exp(mpfr(density.iVU.holder, precBits=53)))))
         }
         
-        cat("\n---------------------------------------------- \n ")
-        cat("Marignal Likelihood Computation Step 2 \n")
-        cat("    density.iVU: ", as.numeric(density.iVU), "\n")
-        cat("---------------------------------------------- \n ")
+        ## cat("\n---------------------------------------------- \n ")
+        ## cat("Marignal Likelihood Computation Step 2 \n")
+        ## cat("    density.iVU: ", as.numeric(density.iVU), "\n")
+        ## cat("---------------------------------------------- \n ")
 
 
         ##
@@ -538,8 +577,10 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         for (g in 1:reduce.mcmc){
             
             ## Step 1: update bhat
-            bhat <- updateb(Z, MU, s2, XtX, b0, B0)
-            Zb <- Z - bhat
+            if(constant){
+                bhat <- updateb(Z, MU, s2, XtX, b0, B0)
+                Zb <- Z - bhat
+            }
             
             ## Step 2: update U
             U <- updateU(K, U, V, R, Zb, s2, eU.st, iVU.st)
@@ -582,15 +623,15 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         
         density.eV <- log(mean(exp(density.eV.holder)))
         if(abs(density.eV) == Inf){
-            cat("    Precision reinforced! \n")
-            print(density.eV.holder)
+            ## cat("    Precision enforced! \n")
+            ## print(density.eV.holder)
             density.eV <- as.numeric(log(mean(exp(mpfr(density.eV.holder, precBits=53)))))
         }
         
-        cat("\n---------------------------------------------- \n ")
-        cat("Marignal Likelihood Computation Step 3 \n")
-        cat("    density.eV: ", as.numeric(density.eV), "\n")
-        cat("---------------------------------------------- \n ")
+        ## cat("\n---------------------------------------------- \n ")
+        ## cat("Marignal Likelihood Computation Step 3 \n")
+        ## cat("    density.eV: ", as.numeric(density.eV), "\n")
+        ## cat("---------------------------------------------- \n ")
 
         ##
         ## Marginal Step 4: p(iVV.st|Z, eU.st, iVU.st, eV.st)
@@ -599,8 +640,10 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         for (g in 1:reduce.mcmc){
             
             ## Step 1: update bhat
-            bhat <- updateb(Z, MU, s2, XtX, b0, B0)
-            Zb <- Z - bhat
+            if(constant){
+                bhat <- updateb(Z, MU, s2, XtX, b0, B0)
+                Zb <- Z - bhat
+            }
             
             ## Step 2: update U
             U <- updateU(K, U, V, R, Zb, s2, eU.st, iVU.st)
@@ -643,72 +686,73 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         
         density.iVV <- log(mean(exp(density.iVV.holder)))
         if(abs(density.iVV) == Inf){
-            cat("    Precision reinforced! \n")
-            print(density.iVV.holder)
+            ## cat("    Precision enforced! \n")
+            ## print(density.iVV.holder)
             density.iVV <- log(mean(exp(mpfr(density.iVV.holder, precBits=53))))
         }
         
-        cat("\n---------------------------------------------- \n ")
-        cat("Marignal Likelihood Computation Step 4 \n")
-        cat("    density.iVV: ", as.numeric(density.iVV), "\n")
-        cat("---------------------------------------------- \n ")
+        ## cat("\n---------------------------------------------- \n ")
+        ## cat("Marignal Likelihood Computation Step 4 \n")
+        ## cat("    density.iVV: ", as.numeric(density.iVV), "\n")
+        ## cat("---------------------------------------------- \n ")
 
-        ##
-        ## Marginal Step 5: p(bhat.st|Z, eU.st, iVU.st, eV.st, iVV.st)
-        ##
-        density.bhat.holder <- matrix(NA, reduce.mcmc)
-        for (g in 1:reduce.mcmc){
-            
-            ## Step 1: update bhat
-            ZU <- Z - MU
-            Xtz <- sum(ZU) ## t(apply(X, 4, c))%*%c(ZU)
-            cV <- 1/(XtX/s2 +  1/B0) ## solve( XtX/s2 +  diag(1/B0, p))
-            cE <- cV*(Xtz/s2 + (1/B0)*b0) ## cV%*%(Xtz/s2 + diag(1/B0, p)*b0)
-            bhat <- rnorm(1, cE, sqrt(cV)) ## rMVNorm(1,cE,cV)[1:p]
-            Zb <- Z - bhat
-            density.bhat.holder[g] <- dnorm(bhat.st, cE, sqrt(cV), log=TRUE)            
-            ## bhat <- updateb(Z, MU, s2, XtX, b0, B0)
-            ## Zb <- Z - bhat
-            
-            ## Step 2: update U
-            U <- updateU(K, U, V, R, Zb, s2, eU.st, iVU.st)
-            if (UL.Normal == "Normal"){
-                U <- Unormal(U)
-            }else if(UL.Normal == "Orthonormal"){
-                U <- GramSchmidt(U)
+        if(constant){
+            ##
+            ## Marginal Step 5: p(bhat.st|Z, eU.st, iVU.st, eV.st, iVV.st)
+            ##
+            density.bhat.holder <- matrix(NA, reduce.mcmc)
+            for (g in 1:reduce.mcmc){
+                
+                ## Step 1: update bhat
+                ZU <- Z - MU
+                Xtz <- sum(ZU) ## t(apply(X, 4, c))%*%c(ZU)
+                cV <- 1/(XtX/s2 +  1/B0) ## solve( XtX/s2 +  diag(1/B0, p))
+                cE <- cV*(Xtz/s2 + (1/B0)*b0) ## cV%*%(Xtz/s2 + diag(1/B0, p)*b0)
+                bhat <- rnorm(1, cE, sqrt(cV)) ## rMVNorm(1,cE,cV)[1:p]
+                Zb <- Z - bhat
+                density.bhat.holder[g] <- dnorm(bhat.st, cE, sqrt(cV), log=TRUE)            
+                ## bhat <- updateb(Z, MU, s2, XtX, b0, B0)
+                ## Zb <- Z - bhat
+                
+                ## Step 2: update U
+                U <- updateU(K, U, V, R, Zb, s2, eU.st, iVU.st)
+                if (UL.Normal == "Normal"){
+                    U <- Unormal(U)
+                }else if(UL.Normal == "Orthonormal"){
+                    U <- GramSchmidt(U)
+                }
+                
+                ## Step 3: update V
+                V <- updateV(Zb, U, R, K, s2, eV.st, iVV.st, UTA)         
+                ## update MU
+                MU <- M.U(list(U,U,V))
+                
+                ## Step 4: update eU and iVU
+                ## SS <-  diag(psi0, nrow=R) +  t(U)%*%U 
+                ## iVU <- rwish(solve(SS), Psi0+K[1])
+                ## eU <- c(rMVNorm(1, apply(U,2,sum)/(K[1]+1), solve(iVU)/(K[1]+1)))
+                
+                ## Step 5: update eV and iVV
+                ## SS <-  diag(psi0, nrow=R) +  t(V)%*%V ## (K[3]-1)*cov(V)   +K[3]*msi/(K[3]+1)
+                ## iVV <- rwish(solve(SS), Psi0+K[3])
+                ## eV <- c(rMVNorm(1,apply(V,2,sum)/(K[3]+1), solve(iVV)/(K[3]+1)))
+                
+                ## Step 6: update s2
+                ZE <- Zb - MU 
+                s2 <- rinvgamma(1, (c0 + XtX)/2, (d0+ sum(c(ZE)^2))/2  )        
             }
             
-            ## Step 3: update V
-            V <- updateV(Zb, U, R, K, s2, eV.st, iVV.st, UTA)         
-            ## update MU
-            MU <- M.U(list(U,U,V))
-            
-            ## Step 4: update eU and iVU
-            ## SS <-  diag(psi0, nrow=R) +  t(U)%*%U 
-            ## iVU <- rwish(solve(SS), Psi0+K[1])
-            ## eU <- c(rMVNorm(1, apply(U,2,sum)/(K[1]+1), solve(iVU)/(K[1]+1)))
-            
-            ## Step 5: update eV and iVV
-            ## SS <-  diag(psi0, nrow=R) +  t(V)%*%V ## (K[3]-1)*cov(V)   +K[3]*msi/(K[3]+1)
-            ## iVV <- rwish(solve(SS), Psi0+K[3])
-            ## eV <- c(rMVNorm(1,apply(V,2,sum)/(K[3]+1), solve(iVV)/(K[3]+1)))
-            
-            ## Step 6: update s2
-            ZE <- Zb - MU 
-            s2 <- rinvgamma(1, (c0 + XtX)/2, (d0+ sum(c(ZE)^2))/2  )        
+            density.bhat <- log(mean(exp(density.bhat.holder)))
+            if(abs(density.bhat) == Inf){
+                ## cat("    Precision reinforced! \n")
+                ## print(density.bhat.holder)
+                density.bhat <- as.numeric(log(mean(exp(mpfr(density.bhat.holder, precBits=53)))))
+            }
+            ## cat("\n---------------------------------------------- \n ")
+            ## cat("Marignal Likelihood Computation Step 5 \n")
+            ## cat("    density.bhat: ", as.numeric(density.bhat), "\n")
+            ## cat("---------------------------------------------- \n ")
         }
-        
-        density.bhat <- log(mean(exp(density.bhat.holder)))
-        if(abs(density.bhat) == Inf){
-            cat("    Precision reinforced! \n")
-            print(density.bhat.holder)
-            density.bhat <- as.numeric(log(mean(exp(mpfr(density.bhat.holder, precBits=53)))))
-        }
-        
-        cat("\n---------------------------------------------- \n ")
-        cat("Marignal Likelihood Computation Step 5 \n")
-        cat("    density.bhat: ", as.numeric(density.bhat), "\n")
-        cat("---------------------------------------------- \n ")
 
         ##
         ## Marginal Step 6: p(sigma.st|Z, eU.st, iVU.st, eV.st, iVV.st, bhat.st)
@@ -753,16 +797,16 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         
         density.Sigma <- log(mean(exp(density.Sigma.holder)))
         if(abs(density.Sigma) == Inf){
-            cat("    Precision reinforced! \n")
-            print(density.Sigma.holder)
+            ## cat("    Precision reinforced! \n")
+            ## print(density.Sigma.holder)
             density.Sigma <- as.numeric(log(mean(exp(mpfr(density.Sigma.holder, precBits=53)))))
         }
         
-        cat("\n---------------------------------------------- \n ")
-        cat("Marignal Likelihood Computation Step 6 \n")
-        cat("    density.Sigma: ", as.numeric(density.Sigma), "\n")
-        cat("---------------------------------------------- \n ")
-        
+        ## cat("\n---------------------------------------------- \n ")
+        ## cat("Marignal Likelihood Computation Step 6 \n")
+        ## cat("    density.Sigma: ", as.numeric(density.Sigma), "\n")
+        ## cat("---------------------------------------------- \n ")
+        ## 
         
         ## Prior ordinate
         iVV <- iVU <- diag(R) ; eV <- eU <- rep(0,R) 
@@ -778,24 +822,32 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
         density.iVU.prior <- sum(dens.temp.iVU)
         density.iVV.prior <- sum(dens.temp.iVV)
 
-        density.beta.prior <- dnorm(bhat.st, b0, sqrt(B0), log=TRUE) ## p = 1
+        if(constant){
+            density.beta.prior <- dnorm(bhat.st, b0, sqrt(B0), log=TRUE) ## p = 1
+        }
         density.Sigma.prior <- log(dinvgamma(Sigma.st, c0/2, (d0)/2))
 
 
-        logprior <- density.eU.prior + density.eV.prior + density.iVU.prior + density.iVV.prior +
-            density.beta.prior + density.Sigma.prior
-        logdenom <- (density.eU + density.eV + density.iVU + density.iVV +
-                                                   density.bhat + density.Sigma)
+        if(constant){
+            logprior <- density.eU.prior + density.eV.prior + density.iVU.prior + density.iVV.prior +
+                density.beta.prior + density.Sigma.prior
+            logdenom <- (density.eU + density.eV + density.iVU + density.iVV +
+                         density.bhat + density.Sigma)
+        }else{
+            logprior <- density.eU.prior + density.eV.prior + density.iVU.prior + density.iVV.prior +
+                density.Sigma.prior
+            logdenom <- (density.eU + density.eV + density.iVU + density.iVV + density.Sigma)
+        }
         ## logmarglike <- (loglike + logprior) - logdenom;
         logmarglike.upper <- (loglike.upper + logprior) - logdenom
         
-        cat("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
+        ## cat("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
         ## cat("    log marginal likelihood = (loglike + logprior) - (density.parameters) \n")
         ## cat("    log marginal likelihood : ", as.numeric(logmarglike), "\n")
-        cat("    log marginal likelihood : ", as.numeric(logmarglike.upper), "\n")
-        cat("    logprior: ", as.numeric(logprior), "\n")
-        cat("    log posterior density: ", as.numeric(logdenom), "\n")
-        cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
+        ## cat("    log marginal likelihood : ", as.numeric(logmarglike.upper), "\n")
+        ## cat("    log prior: ", as.numeric(logprior), "\n")
+        ## cat("    log posterior density: ", as.numeric(logdenom), "\n")
+        ## cat("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ \n")
                 
     }
     attr(output, "title") <- "NetworkStatic Posterior Sample"
@@ -807,9 +859,12 @@ NetworkStatic <- function(Y, R=2, mcmc = 100, burnin = 100, verbose = 0,thin = 1
     attr(output, "iVV") <- iVVmat
     attr(output, "U") <- U
     attr(output, "V") <- V
+    attr(output, "MU") <- MU.record/nss
     attr(output, "Umat") <- Umat
     attr(output, "Vmat") <- Vmat
-    attr(output, "bmat") <- bmat
+    if(constant){
+        attr(output, "bmat") <- bmat
+    }
     attr(output, "s2mat") <- s2mat
     attr(output, "mcmc") <- nstore
     attr(output, "R") <- R
